@@ -1,3 +1,5 @@
+# FOR CONNECTION TO FLASHCAM AND NOTIFATOR - SEE FLASHCAM README.org
+
 import cv2
 from flashcam.base_camera2 import BaseCamera
 
@@ -26,6 +28,13 @@ from flashcam.mmapwr import mmread_n_clear, mmread
 import os
 import sys
 
+from notifator import telegram
+import threading
+
+try:
+    import pyautogui # take screenshot
+except:
+    print("X... no DISPLAY, pyautogui cannot be used")
 # -----------------------------------------------------------------
 
 # -----------------------------------------------------------------
@@ -41,23 +50,20 @@ class Camera(BaseCamera):
         should return videocapture device
         but also sould set Camerare.video_source
         """
-
-        # ----------------------------
-        #    we need to get in into the thread.....
-        #  NOT NOW - all is taken from BaseCam
-        # def __init__(self, target_frame = "direct" , average = 0, blur = 0 , threshold = 0):
-
-
+        #  - all is taken from BaseCam
         # res = "640x480"
-        res = config.CONFIG["res"]
-        print("D... init_cam caleld with:", res )
-        print("i... init_cam caleld with:", res )
-        print("D... init_cam caleld with:",  config.CONFIG["product"] )
-        print("i... init_cam caleld with:",  config.CONFIG["product"] )
+        res = config.CONFIG["resolution"]
+        print("D... init_cam caleld with res:", res )
+        #print("i... init_cam caleld with prod:", res )
+        print("D... init_cam caleld with prod:",  config.CONFIG["product"] )
+        #print("i... init_cam caleld with prod:",  config.CONFIG["product"] )
 
         vids = recommend_video( config.CONFIG["product"]  )
 
         if len(vids)>0:
+            if vids[0]==-1:
+                return config.CONFIG["product"] , -1
+
             vidnum = vids[0]
             cap = cv2.VideoCapture(vidnum,  cv2.CAP_V4L2)
 
@@ -70,12 +76,13 @@ class Camera(BaseCamera):
 
 
             pixelformat = "MJPG"
-            pixelformat = "YUYV"
+            pixelformat = "YUYV" # I use lossless format for camera readout
+
             w,h =  int(res.split("x")[0]), int(res.split("x")[1])
             fourcc = cv2.VideoWriter_fourcc(*pixelformat)
             cap.set(cv2.CAP_PROP_FOURCC, fourcc)
-            cap.set(cv2
-                    .CAP_PROP_FRAME_WIDTH,   w )
+            print(f"i... setting RESOLUTION: {w} x {h} and PIXELFORMAT {fourcc}")
+            cap.set(cv2.CAP_PROP_FRAME_WIDTH,   w )
             cap.set(cv2.CAP_PROP_FRAME_HEIGHT,  h )
             return cap,vidnum
         return None, None
@@ -87,8 +94,10 @@ class Camera(BaseCamera):
         product= ... uses the recommend_video to restart the same cam
         """
         # i need these to be in globals() ----evaluate from web.py
+        #                                 ---- OR FROM seread
         global substract_background,save_background
         global mix_foreground,save_foreground
+        global send_telegram, telegramlast
 
         global speedx, speedy, restart_translate, average
         global gamma_divide, gamma_multiply,gamma_setdef
@@ -116,7 +125,7 @@ class Camera(BaseCamera):
         blur         = int(config.CONFIG['blur'])
         timelaps     = int(config.CONFIG['laps'])
         histogram    = config.CONFIG['histogram']
-        res          = config.CONFIG['res']
+        res          = config.CONFIG['resolution']
         speedx       = float(config.CONFIG['x'])
         speedy       = float(config.CONFIG['y'])
         rotate180    = int(config.CONFIG['otate'])
@@ -131,11 +140,17 @@ class Camera(BaseCamera):
         print( "XY: ", config.CONFIG['x'] ,  config.CONFIG['y']  , speedx, speedy)
 
         # ------------------    to evaluate commands from web.py
+        # ------------------    or searead
+        # ------------------    these commands need to be declared here
+        #                       AND in globals
         substract_background = False
         save_background = False
         mix_foreground = False
         save_foreground = False
 
+        send_telegram = None # i hope this is ok too...
+        telegramlast = datetime.datetime.now()
+
         restart_translate = False
 
         gamma_divide = False
@@ -158,26 +173,67 @@ class Camera(BaseCamera):
         fixed_image = None # just camera.
 
         camera = Camera(  )
-        vidnum = None
-        while vidnum is None:
-            cap, vidnum = camera.init_cam(  )
-            if vidnum is None:
-                print("!... no video recommended, locked in a loop (realcam/frames)")
-                fullpath_fixed_image = "~/.config/flashcam/monoskop.jpg"
-                fullpath_fixed_image = os.path.expanduser( fullpath_fixed_image )
+        vidnum = None # it will be re-asked gain and again
+
+        while (vidnum is None) or (vidnum == -1):
+            cap, vidnum = camera.init_cam(  ) # can return None,None
+
+
+            fullpath_fixed_image = "~/.config/flashcam/monoskop.jpg"
+            fullpath_fixed_image = os.path.expanduser( fullpath_fixed_image)
+            if not (vidnum is None) and (vidnum==-1):
+                if (cap.find("screenshot.jpg")==0) and ('pyautogui' in globals()):
+                    image = pyautogui.screenshot()
+                    image = cv2.cvtColor(np.array(image), cv2.COLOR_RGB2BGR)
+
+                    scale_percent = 50
+                    width = int(image.shape[1] * scale_percent / 100)
+                    height = int(image.shape[0] * scale_percent / 100)
+                    # dsize
+                    dsize = (width, height)
+                    # resize image
+                    image = cv2.resize(image, dsize)
+                    frame = image
+                    #cv2.imwrite("screenshot.jpg", image)
+                    #print("i... screenshot.jpg")
+                    fullpath_fixed_image = "screenshot.jpg"
+
+                elif cap.find(".jpg")>=0:
+                    #print(cap)
+                    #print(cap)
+                    fullpath_fixed_image = f"{cap}"
+                    fullpath_fixed_image = os.path.expanduser( fullpath_fixed_image )
+                    if not os.path.exists(fullpath_fixed_image):
+                        fullpath_fixed_image = f"~/.config/flashcam/{cap}"
+                        fullpath_fixed_image = os.path.expanduser( fullpath_fixed_image )
+                        if not os.path.exists(fullpath_fixed_image):
+                            fullpath_fixed_image = f"~/.config/flashcam/monoskop.jpg"
+                            fullpath_fixed_image = os.path.expanduser( fullpath_fixed_image)
+
+
+            if (vidnum is None) or (vidnum==-1):
+                print(f"!... no VID recommended, I'm in a loop (realcam/frames) {datetime.datetime.now().strftime('%H:%M:%S')}", end = "\r")
+
+                #print(fullpath_fixed_image)
+                # fullpath_fixed_image = "~/.config/flashcam/monoskop.jpg"
+
                 if os.path.exists(fullpath_fixed_image):
                     frame = cv2.imread( fullpath_fixed_image)
-                    print(f"i...  {fullpath_fixed_image}")
+                    #print(f"i...  {fullpath_fixed_image}")
                     senh.add_frame(frame)
                     senh.setbox(" ", senh.TIME)
                     frame = senh.get_frame(  )
                     # this is not giving anything
                     yield frame
+                elif fullpath_fixed_image=="screenshot.jpg":
+                    senh.add_frame(frame)
+                    senh.setbox(" ", senh.TIME)
+                    frame = senh.get_frame(  )
+                    yield frame
                 else:
-                    print(f"!... monoskop.jpg not found {fullpath_fixed_image}")
+                    print(f"!... broadcast image not found: {fullpath_fixed_image}")
                 time.sleep(1)
 
-
         #=------------------------------------------- EXPO GAIN
 
         cc = v4lc.V4L2_CTL("/dev/video"+str(vidnum))
@@ -264,6 +320,7 @@ class Camera(BaseCamera):
                     config.CONFIG["camera_on"] = False
 
 
+            # --- camera probably works ret True
             if not ret:
                 time.sleep(0.5)
                 config.CONFIG["camera_on"] = False
@@ -290,8 +347,12 @@ class Camera(BaseCamera):
                     print("X... prev_frame was bad, no gray image")
 
             #print("D... ret==", ret)
+
+
             if ret: #********************************************************* operations
                 # fixed_image = "beamon.jpg"
+                # ----- probably mmap COMMAND FROM seread "fixed_image imgname"
+                #
                 if not(fixed_image is None):
                     fullpath_fixed_image = "~/.config/flashcam/"+fixed_image
                     fullpath_fixed_image = os.path.expanduser( fullpath_fixed_image )
@@ -299,6 +360,9 @@ class Camera(BaseCamera):
                         frame = cv2.imread( fullpath_fixed_image)
 
                 frame_prev = frame
+
+
+
                 if senh.add_frame(frame):  # it is a proper image....
 
 
@@ -382,8 +446,9 @@ class Camera(BaseCamera):
 
 
                     # ------------------------- commands comming from web.py----
-
+                    #           -------------- or from seread (fixed_image ...)
                     expression,value = mmread_n_clear( )
+
                     if expression[:5] != "xxxxx":
                         print(f"i...  *  EXPR: {expression} == {value}")
                         print(f"i...  *  EXPR: {expression} == {value}")
@@ -396,6 +461,19 @@ class Camera(BaseCamera):
                         print("X... EXEC FAIL",expression,value)
 
 
+
+                    telegramnow = datetime.datetime.now()
+                    if not (send_telegram is None):
+                        # telegram
+                        if os.path.exists(os.path.expanduser("~/.telegram.token")):
+                            if (telegramnow-telegramlast).total_seconds()>10:
+                                y = threading.Thread(target=telegram.bot_send, args=("ALERT", f"{send_telegram} {telegramnow.strftime('%a %H:%M:%S')}",frame) )
+                                y.start()
+
+                                telegramlast=telegramnow
+                                send_telegram = None
+
+
                     if save_background:
                         print("D... HERE I SAVE save_background of mask")
                         print("D... HERE I SAVE save_background of mask")
@@ -654,26 +732,26 @@ class Camera(BaseCamera):
 
 
 
-    @staticmethod
-    def set_video_source(source):
-        """
-        never user whatsoever !
-        """
-        print("D... set_video_source: source=", source)
-        camera = cv2.VideoCapture( source,  cv2.CAP_V4L2)
-        print("D... ",camera)
-        print("D... setting MJPG writer....FMP4 works too")
-        # camera.set(cv2.CAP_PROP_FOURCC, cv2.VideoWriter_fourcc('M', 'J', 'P', 'G'))
-        camera.set(cv2.CAP_PROP_FOURCC, cv2.VideoWriter_fourcc('F','M','P','4'))
-        print("D... first camera read ....")
-        ok = False
-        try:
-            _, img = camera.read()
-            print(img.size) # this can fail and reset to DEV 0
-            ok = True
-        except Exception as ex:
-            print("X... CAMERA read ... FAILED",ex)
+    # @staticmethod
+    # def set_video_source(source):
+    #     """
+    #     never user whatsoever !
+    #     """
+    #     print("D... set_video_source: source=", source)
+    #     camera = cv2.VideoCapture( source,  cv2.CAP_V4L2)
+    #     print("D... ",camera)
+    #     print("D... setting MJPG writer....FMP4 works too")
+    #     # camera.set(cv2.CAP_PROP_FOURCC, cv2.VideoWriter_fourcc('M', 'J', 'P', 'G'))
+    #     camera.set(cv2.CAP_PROP_FOURCC, cv2.VideoWriter_fourcc('F','M','P','4'))
+    #     print("D... first camera read ....")
+    #     ok = False
+    #     try:
+    #         _, img = camera.read()
+    #         print(img.size) # this can fail and reset to DEV 0
+    #         ok = True
+    #     except Exception as ex:
+    #         print("X... CAMERA read ... FAILED",ex)
 
-        if ok:
-            return camera
-        return None
+    #     if ok:
+    #         return camera
+    #     return None
