@@ -73,8 +73,8 @@ show_help = False
 
 
 # x ... expand 2x ... buggy
-HELPTEXT=""" a/A ... SAVE AVI/ stop (to ~/DATA/)
- s ... save JPG (to ~/DATA/)
+HELPTEXT=""" a/A ... SAVE AVI/stop (~/DATA/) (c-a PNG)
+ s/S ... save 1 JPG (PNG) (to ~/DATA/)
 
  z ... cycle zoom 2x (mouse click fixes the center)
  Z ... zoom 1x
@@ -157,7 +157,14 @@ def remap_keys(key):
              1114200:'X',
              1114201:'Y',
              1114202:'Z'   }
-# ctrl e     1310821:''
+# 1.  I deactivate ALL above keys to avoid duplicity for arrows etc...
+#
+#
+# 2. I use these ctrl-
+# ctrle     1310821:''
+# (key == 1310823): # ctrlg
+#         1310841): #ctrly
+# ctrla
 
     if key in table:
         key = ord( table[key] )
@@ -320,7 +327,7 @@ def img_estim(img, thrshld=127):
 
 def display2(videodev, save = False,
              passfile="~/.pycamfw_userpass",
-             rotate = 0, vof = "99,2"):
+             rotate = 0, vof = "99,2",  savepngcont = False ):
     """
     """
     #sname,sfilenamea,sme,sfilenamea,sfilenamea,sfourcc,saviout
@@ -351,7 +358,39 @@ def display2(videodev, save = False,
             #cv2.imshow('image', imgCopy)
 
 
-    def setupsave():
+
+    """
+
+
+{
+“avi” : [ “avc1”, “DIVX”, “H264”, “X264”, “V264”, “IYUV”, “MJPG”, “MPEG”, “MP42”, “mp4v”, “XVID” ],
+“mov” : [ “avc1”, “DIVX”, “mp4v”, “XVID” ],
+“mp4” : [ “avc1”, “mp4v” ],
+“mkv” : [ “avc1”, “DIVX”, “H264”, “X264”, “V264”, “IYUV”, “MJPG”, “MPEG”, “MP42”, “mp4v”, “XVID” ],
+“3gp” : [ “avc1”, “mp4v” ]
+}
+
+
+Notes:
+– avc1 is equivalent/identical to H264, X264, V264
+– mp4v is the same as DIVX, XVID
+
+File size & compression
+– most compact format: avc1
+– least compact format is IYUV (by a long shot), followed by MPJPG/MPEG
+– middle/similar: MP42, mp4v
+
+Recommendation:
+– use avi or mkv for best compatibility across all codecs for the maximum flexibility
+– use mp4 (avc1, mp4v) for compatibility across players with most/reasonable compression
+
+"""
+
+
+
+
+
+    def setupsave( resolution = (640,480) ):
         sname = "rec"
         sname = videodev
         sname = sname.replace("http","")
@@ -362,7 +401,14 @@ def display2(videodev, save = False,
 
         sfilenamea = datetime.datetime.now().strftime("%Y%m%d_%H%M%S")
         sme = socket.gethostname()
-        sfilenamea = f"{sme}_{sname}_{sfilenamea}.avi"
+        sfilenamea = f"{sme}_{sname}_{sfilenamea}"
+        #
+        # DEFINE EXTENSION : avi+xvid     mov  mp4 mpg
+        #   avi+x264
+        #
+        #sfilenamea = f"{sfilenamea}.mp4"
+        #sfilenamea = f"{sfilenamea}.mp4"
+        sfilenamea = f"{sfilenamea}.mkv"
 
         dir2create = os.path.expanduser("~/DATA/")
         if not os.path.isdir( os.path.expanduser(dir2create )):
@@ -371,8 +417,14 @@ def display2(videodev, save = False,
             os.mkdir( os.path.expanduser(dir2create ))
 
         sfilenamea = os.path.expanduser("~/DATA/" + sfilenamea)
+        # codec
         sfourcc = cv2.VideoWriter_fourcc(*'XVID')
-        saviout = cv2.VideoWriter( sfilenamea,sfourcc,25.0, (640,480))
+        # sfourcc = cv2.VideoWriter_fourcc(*'LAGS') # lossless NOT mkv,avi..,
+        sfourcc = cv2.VideoWriter_fourcc(*'X264') # +avi small
+        #sfourcc = cv2.VideoWriter_fourcc(*'MP4V')
+        #sfourcc = cv2.VideoWriter_fourcc(*'IYUV') # huge + avi
+        #  25 FPS
+        saviout = cv2.VideoWriter( sfilenamea,sfourcc,25.0, resolution )
         print( sfilenamea )
         print( sfilenamea )
         print( sfilenamea )
@@ -544,7 +596,7 @@ def display2(videodev, save = False,
                     #bytex+=b""
                     time.sleep(0.2)
                     io_none+=1
-                    if io_none>20:
+                    if io_none>50:
                         stream = None
                         print("X... ---------------  too many unseen frames", io_none, "breaking")
                         io_none = 0
@@ -555,14 +607,18 @@ def display2(videodev, save = False,
             if stream is None:
                 continue
         else:                     # from file ---- 1st moment open
+            # replay video from file
             ret_val = 0
             stream = cv2.VideoCapture( videodev )
             filesource = True
-            print ("X... open  = ",stream.isOpened())
+            print ("i... FILE open  = ",stream.isOpened())
             ok,frame = stream.read()
             frame_num+=1
             # pause = True too early.... defined later
             print(f"X... {ok}, frame from {videodev}")
+            #print("i... stream in locals() == ", 'stream' in locals() )
+            #print("i... stream is None     == ", stream is None, stream )
+            print("i...   PAUSED: press SPACE to play")
 
 #----------------------------------------------------------------
 
@@ -592,7 +648,7 @@ def display2(videodev, save = False,
         frame_num = 0
 
         senh = Stream_Enhancer()
-        saved_jpg = False
+        saved_jpg = 0  #  0, 1jpg, 2png
 
         zoomme=1
         centerX1, centerY1 = 320,240
@@ -796,7 +852,12 @@ def display2(videodev, save = False,
                         f.write( f"{webframetime} {webframen} {cx} {cy}\n" )
                         #   f.write( f"{webframetime[11:]} {webframen} {cx} {cy}\n" )
 
-                    if (webframen.strip(" ")==""):
+                    # there is time sent from server... by a trick
+                    #
+                    # if from file(or elsewhere) it is int or nothing...
+                    #
+                    if (type(webframen)==int) or (webframen.strip(" ")==""):
+                        # no acq timestamp info from server
                         webframen=frame_num
                         ttime = int(webframen)
                     else:
@@ -928,18 +989,45 @@ def display2(videodev, save = False,
                     #frame = cv2.rotate(frame, cv2.ROTATE_90_CLOCKWISE)
 
 
+                # I just need to put local gamma before avisave...
+                # else astroimage shows nothing..
+                if local_gamma!=1:
+                    frame = adjust_gamma(frame, local_gamma)
+
+
+                if savepngcont:
+                    sfilename = datetime.datetime.now().strftime("%Y%m%d_%H%M%S")
+                    sname = "snapshot"
+                    sfilename = f"{sme}_{sfilename}_{sname}.png"
+                    sfilename = os.path.expanduser( "~/DATA/"+sfilename )
+                    print(f"i... PNGcontinuous {sfilename}")
+
+                    cv2.imwrite(sfilename,frame)
+                    if senh.add_frame(frame):
+                        # print("avi..")
+                        senh.setbox(f"png",  senh.jpg)
+                        frame = senh.get_frame(  )
+
                 if save:
-                    # print("AVI")
+                    #savepngcont = False
+                    print("AVI")  # OR MKV now with H264
                     saviout.write(frame)
                     if senh.add_frame(frame):
                         # print("avi..")
                         senh.setbox(f"AVI",  senh.avi)
                         frame = senh.get_frame(  )
-                if saved_jpg:
+
+
+                if saved_jpg==1:
                     if senh.add_frame(frame):
                         # print("avi..")
                         senh.setbox(f"JPG",  senh.jpg)
                         frame = senh.get_frame(  )
+                if saved_jpg==2:
+                    if senh.add_frame(frame):
+                        # print("avi..")
+                        senh.setbox(f"PNG",  senh.jpg)
+                        frame = senh.get_frame(  )
 
 
 
@@ -1166,8 +1254,6 @@ T ... use tracker2
 
 
 
-                if local_gamma!=1:
-                    frame = adjust_gamma(frame, local_gamma)
 
 
 
@@ -1251,7 +1337,7 @@ in progress"""
                     webbrowser.open( videodev.replace("/video","" ) ) # BRUTAL
 
 
-                if (not frame is None) and (rpi_name!="") and (key == ord(' ')):
+                if (not frame is None) and (rpi_name!="") and ( (key==1048608) or (key == ord(' '))):
                     print("SPC PRESSED! - pause/play")
                     pause = not pause
                     print(f" ... pause = {pause}" )
@@ -1274,24 +1360,38 @@ in progress"""
 
 
                 if (not frame is None) and (rpi_name!="") and (key == ord('a')):
-                    print("a PRESSED! - saving AVI")
+                    print("a PRESSED! - saving AVI (mkv)")
                     save = True
-                    sfilenamea,saviout = setupsave()
+
+                    height, width, channel = frame.shape
+
+                    sfilenamea,saviout = setupsave( (width,height)  )
                     print(">>>", sfilenamea )
 
+
+                if (not frame is None) and (rpi_name!="") and (key == 1310817):
+                    print("ctrl-a PRESSED! - saving PNG cont") # ctrla
+                    savepngcont = True
+
                 if (not frame is None) and (rpi_name!="") and (key == ord('A')):
-                    print("A PRESSED! - STOPPING stopping saving AVI")
+                    print("A PRESSED! - STOPPING stopping saving AVI (mkv)")
                     save = False
                     sfilenamea = ""
+                    savepngcont = False
 
-                saved_jpg = False
-                if (not frame is None) and (rpi_name!="") and (key == ord('s')):
-                    print("s PRESSED!")
+
+                saved_jpg = 0
+                if (not frame is None) and (rpi_name!="") and  ((key == ord('s')) or (key == ord('S')  ) ):
+                    print("s or S PRESSED!")
                     sname = "snapshot"
-                    saved_jpg = True
                     sfilename = datetime.datetime.now().strftime("%Y%m%d_%H%M%S")
                     # defined above # sme = socket.gethostname()
-                    sfilename = f"{sme}{sname}_{sfilename}.jpg"
+                    if key == ord('s'):
+                        sfilename = f"{sme}_{sfilename}_{sname}.jpg"
+                        saved_jpg = 1
+                    if key == ord('S'):
+                        sfilename = f"{sme}_{sfilename}_{sname}.png"
+                        saved_jpg = 2
 
                     dir2create = os.path.expanduser("~/DATA/")
                     if not os.path.isdir( os.path.expanduser(dir2create )):
@@ -1313,6 +1413,7 @@ in progress"""
                 if (not frame is None) and (rpi_name!=""):
                     if (key == ord('C')):
                         print(f"c PRESSED! - cross = {cross} OFF; saving {FILE_REDCROSS}")
+                        if (cross is None) or (len(cross)!=2): break
                         with open( os.path.expanduser(FILE_REDCROSS) ,"w") as f:
                             f.write( f"{cross[0]}\n{cross[1]}\n" )
                             cross = None
@@ -1705,7 +1806,7 @@ in progress"""
                     post_response = requests.post(url=post_addr, data=post_data)
 
                 if (not frame is None) and (rpi_name!="") \
-                   and  (key == 1310821 ):
+                   and  (key == 1310821 ): # ctrle
                     print("ctrl-e PRESSED! - " )
                     post_addr = videodev.replace("/video","/cross" )
                     post_data = {'expo':'EXPO'}
@@ -1729,7 +1830,7 @@ in progress"""
                     post_response = requests.post(url=post_addr, data=post_data)
 
                 if (not frame is None) and (rpi_name!="") \
-                   and  (key == 1310823):
+                   and  (key == 1310823): # ctrlg
                     print("ctrl-g PRESSED! - " )
                     post_addr = videodev.replace("/video","/cross" )
                     post_data = {'gain':'GAIN'}
@@ -1749,7 +1850,7 @@ in progress"""
                     local_gamma = local_gamma /1.4
 
                 if (not frame is None) and (rpi_name!="") \
-                   and  (key == 1310841):
+                   and  (key == 1310841): #ctrly
                     print("ctrl-y PRESSED! - " )
                     local_gamma = 1
 
@@ -1815,9 +1916,9 @@ in progress"""
                     post_response = requests.post(url=post_addr, data=post_data)
 
 
-
-#                if key!=-1:
-#                    print(f"\n{key}\n")
+                # TEST key code
+                if key!=-1:
+                    print(f"\n{key}\n")
 
 if __name__=="__main__":
     Fire( display3)
